@@ -97,6 +97,10 @@
 
 static int force_ca_check;
 static int check_keyring;
+static int opt_verbose_listing;
+static int opt_list_all;
+
+static const char* const db_names[] = { "MokListRT", "MokListXRT", "PK", "KEK", "db", "dbx" };
 
 typedef struct {
 	uint32_t mok_toggle_state;
@@ -111,23 +115,23 @@ print_help ()
 	printf ("  mokutil OPTIONS [ARGS...]\n");
 	printf ("\n");
 	printf ("Options:\n");
-	printf ("  --help\t\t\t\tShow help\n");
-	printf ("  --list-enrolled\t\t\tList the enrolled keys\n");
-	printf ("  --list-new\t\t\t\tList the keys to be enrolled\n");
-	printf ("  --list-delete\t\t\t\tList the keys to be deleted\n");
-	printf ("  --import <der file...>\t\tImport keys\n");
-	printf ("  --delete <der file...>\t\tDelete specific keys\n");
+	printf ("  --help, -h\t\t\t\tShow help\n");
+	printf ("  --list-enrolled, -l\t\t\tList the enrolled keys\n");
+	printf ("  --list-new, -N\t\t\tList the keys to be enrolled\n");
+	printf ("  --list-delete, -D\t\t\tList the keys to be deleted\n");
+	printf ("  --import, -i <der file...>\t\tImport keys\n");
+	printf ("  --delete, -D <der file...>\t\tDelete specific keys\n");
 	printf ("  --revoke-import\t\t\tRevoke the import request\n");
 	printf ("  --revoke-delete\t\t\tRevoke the delete request\n");
-	printf ("  --export\t\t\t\tExport keys to files\n");
-	printf ("  --password\t\t\t\tSet MOK password\n");
-	printf ("  --clear-password\t\t\tClear MOK password\n");
+	printf ("  --export, -x\t\t\t\tExport keys to files\n");
+	printf ("  --password, -p\t\t\tSet MOK password\n");
+	printf ("  --clear-password, -c\t\t\tClear MOK password\n");
 	printf ("  --disable-validation\t\t\tDisable signature validation\n");
 	printf ("  --enable-validation\t\t\tEnable signature validation\n");
 	printf ("  --sb-state\t\t\t\tShow SecureBoot State\n");
-	printf ("  --test-key <der file>\t\t\tTest if the key is enrolled or not\n");
+	printf ("  --test-key, -t <der file>\t\tTest if the key is enrolled or not\n");
 	printf ("  --reset\t\t\t\tReset MOK list\n");
-	printf ("  --generate-hash[=password]\t\tGenerate the password hash\n");
+	printf ("  --generate-hash[=password], -g\tGenerate the password hash\n");
 	printf ("  --ignore-db\t\t\t\tIgnore DB for validation\n");
 	printf ("  --use-db\t\t\t\tUse DB for validation\n");
 	printf ("  --import-hash <hash>\t\t\tImport a hash into MOK or MOKX\n");
@@ -151,11 +155,14 @@ print_help ()
 	printf ("  --list-sbat-revocations\t\tList the entries in SBAT\n");
 	printf ("\n");
 	printf ("Supplimentary Options:\n");
-	printf ("  --hash-file <hash file>\t\tUse the specific password hash\n");
-	printf ("  --root-pw\t\t\t\tUse the root password\n");
-	printf ("  --mokx\t\t\t\tManipulate the MOK blacklist\n");
+	printf ("  --hash-file, -f <hash file>\t\tUse the specific password hash\n");
+	printf ("  --root-pw, -P\t\t\t\tUse the root password\n");
+	printf ("  --mok, -m\t\t\t\tManipulate the MOK list\n");
+	printf ("  --mokx, -X\t\t\t\tManipulate the MOK blacklist\n");
 	printf ("  --ca-check\t\t\t\tCheck if CA of the key is enrolled/blocked\n");
 	printf ("  --ignore-keyring\t\t\tDon't check if the key is the kernel keyring\n");
+	printf ("  --verbose-listing\t\t\tWhen listing keys print them with more detail\n");
+	printf ("  --all, -a\t\t\t\tWhen listing keys print all databases\n");
 }
 
 static int
@@ -172,22 +179,24 @@ list_keys (const uint8_t *data, const size_t data_size)
 	for (unsigned int i = 0; i < mok_num; i++) {
 		char *owner_str = NULL;
 		int ret;
-		printf ("[key %d]\n", i+1);
+		if (opt_verbose_listing) {
+			printf ("[key %d]\n", i+1);
 
-		ret = efi_guid_to_str(&list[i].owner, &owner_str);
-		if (ret > 0) {
-			printf ("Owner: %s\n", owner_str);
-			free (owner_str);
+			ret = efi_guid_to_str(&list[i].owner, &owner_str);
+			if (ret > 0) {
+				printf ("Owner: %s\n", owner_str);
+				free (owner_str);
+			}
 		}
 
 		efi_guid_t sigtype = list[i].header->SignatureType;
 		if (efi_guid_cmp (&sigtype, &efi_guid_x509_cert) == 0) {
-			print_x509 (list[i].mok, list[i].mok_size);
+			print_x509 (list[i].mok, list[i].mok_size, opt_verbose_listing);
 		} else {
 			print_hash_array (&sigtype,
-					  list[i].mok, list[i].mok_size);
+					  list[i].mok, list[i].mok_size, opt_verbose_listing);
 		}
-		if (i < mok_num - 1)
+		if (opt_verbose_listing && i < mok_num - 1)
 			printf ("\n");
 	}
 
@@ -397,7 +406,8 @@ get_pw_hash_from_file (const char *file, pw_crypt_t *pw_crypt)
 	bzero (string, BUF_SIZE);
 
 	while (read_len < BUF_SIZE) {
-		ssize_t rc = read (fd, string + read_len, BUF_SIZE - read_len);
+		ssize_t rc = read (fd, string + read_len,
+				   BUF_SIZE - read_len - 1);
 		if (rc < 0) {
 			if (errno == EINTR || errno == EAGAIN)
 				continue;
@@ -412,11 +422,6 @@ get_pw_hash_from_file (const char *file, pw_crypt_t *pw_crypt)
 	}
 	close (fd);
 
-	if (string[read_len] != '\0') {
-		fprintf (stderr, "corrupted string\n");
-		return -1;
-	}
-
 	if (decode_pass (string, pw_crypt) < 0) {
 		fprintf (stderr, "Failed to parse the string\n");
 		return -1;
@@ -1220,6 +1225,8 @@ export_db_keys (const DBName db_name)
 		case DBX:
 			guid = efi_guid_security;
 			break;
+		case _DB_NAME_MAX:
+			return -1;
 	};
 
 	db_var_name = get_db_var_name(db_name);
@@ -1774,6 +1781,8 @@ list_db (const DBName db_name)
 			return list_keys_in_var ("db", efi_guid_security);
 		case DBX:
 			return list_keys_in_var ("dbx", efi_guid_security);
+		case _DB_NAME_MAX:
+			return -1;
 	}
 
 	return -1;
@@ -1873,11 +1882,13 @@ main (int argc, char *argv[])
 			{"ca-check",           no_argument,       0, 0  },
 			{"ignore-keyring",     no_argument,       0, 0  },
 			{"version",            no_argument,       0, 'v'},
+			{"verbose-listing",    no_argument,       0, 0  },
+			{"all",                no_argument,       0, 'a'},
 			{0, 0, 0, 0}
 		};
 
 		int option_index = 0;
-		c = getopt_long (argc, argv, "cd:f:g::hi:lmpt:xDNPXv",
+		c = getopt_long (argc, argv, "acd:f:g::hi:lmpt:xDNPXv",
 				 long_options, &option_index);
 
 		if (c == -1)
@@ -2009,8 +2020,13 @@ main (int argc, char *argv[])
 				force_ca_check = 1;
 			} else if (strcmp (option, "ignore-keyring") == 0) {
 				check_keyring = 0;
+			} else if (strcmp (option, "verbose-listing") == 0) {
+				opt_verbose_listing = 1;
 			}
 
+			break;
+		case 'a':
+			opt_list_all = 1;
 			break;
 		case 'l':
 			command |= LIST_ENROLLED;
@@ -2170,7 +2186,18 @@ main (int argc, char *argv[])
 	switch (command) {
 		case LIST_ENROLLED:
 		case LIST_ENROLLED | MOKX:
-			ret = list_db (db_name);
+			if (opt_list_all) {
+				ret = 0;
+				for (DBName db = MOK_LIST_RT; db < _DB_NAME_MAX; ++db) {
+					int r;
+					printf("[%s]\n", db_names[db]);
+					r = list_db (db);
+					if (r)
+						ret = r;
+				}
+			} else {
+				ret = list_db (db_name);
+			}
 			break;
 		case LIST_NEW:
 			ret = list_keys_in_var ("MokNew", efi_guid_shim);
